#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>

#include <gdbm.h>
#include <glib.h>



int main (int argc, char** argv)
{
    GDBM_FILE dbf;
    datum key, data, nextkey;
    unsigned int *k;
    char* dir;
    struct stat st;
    char* dir_buf = NULL;

    if (argc != 2 && argc != 3) {
        printf ("Usage: dbm_dump file.db [dir_name]\n");
        return 0;
    }

    dir = argc == 3 ? argv[2] : NULL;

    /* check that this is a directory */
    if (dir) {
        if (stat (dir, &st)) {
            printf ("Directory to store files not accessible\n");
            return 0;
        } 
        else {
            if (!S_ISDIR (st.st_mode)) {
                printf ("Second argument is not a directory\n");
                return 0;
            }
        }
    }

    dbf = gdbm_open(argv[1], 0, GDBM_READER, 0, NULL);

    if (!dbf) {
        printf ("Cannot open file\n");
        return 0;
    }

    if (dir) {
        dir_buf = (char*)calloc (strlen (dir)+256, 1);
        if (!dir_buf) {
            printf ("Memory allocation error\n");
            gdbm_close (dbf);
            return 0;
        }
    }

    key = gdbm_firstkey (dbf);

    while (key.dptr) {
        k = (unsigned int*)key.dptr;
        nextkey = gdbm_nextkey (dbf, key);

        printf ("S: %d, Z: %u, X: %u, Y: %u\n", key.dsize, GINT_FROM_BE (k[0]), GINT_FROM_BE (k[1]), GINT_FROM_BE (k[2]));

        if (dir) {
            data = gdbm_fetch (dbf, key);
            if (!data.dptr)
                printf ("Error: Cannot fetch this entry: %s\n", gdbm_strerror(gdbm_errno));
            else {
                int fd;

                sprintf (dir_buf, "%s/%d_%d_%d.png", dir, GINT_FROM_BE (k[1]), GINT_FROM_BE (k[2]), GINT_FROM_BE (k[0]));
                fd = open (dir_buf, O_CREAT | O_TRUNC | O_WRONLY, S_IREAD | S_IWRITE | S_IRGRP | S_IROTH);
                if (write (fd, data.dptr, data.dsize) == data.dsize)
                    printf ("%s: OK\n", dir_buf);
                else
                    printf ("%s: Error\n", dir_buf);
                close (fd);
                free (data.dptr);
            }
        }

        free (key.dptr);
        key = nextkey;
    }
        
    if (dir_buf)
        free (dir_buf);
    gdbm_close (dbf);

    return 0;
}
