#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <dirent.h>
#include <string.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <gdbm.h>
#include <glib.h>


int main (int argc, char** argv)
{
    GDBM_FILE dbf;
    datum key, data;
    unsigned int *k;
    DIR* dir;
    struct dirent* entry;
    char* p;
    int x, y, z;
    char c1, c2;
    char* name_buf;
    struct stat st;
    char* buf;
    int fd;

    if (argc != 3) {
        printf ("Usage: dbm_update file.db image_dir\n");
        return 0;
    }

    dbf = gdbm_open(argv[1], 0, GDBM_WRCREAT, S_IREAD | S_IWRITE | S_IRGRP | S_IROTH, NULL);

    if (!dbf) {
        printf ("Cannot open file\n");
        return 0;
    }

    dir = opendir (argv[2]);

    if (!dir) {
        gdbm_close (dbf);
        printf ("Cannot read directory contents\n");
        return 0;
    }
    
    if (!(name_buf = (char*)calloc (strlen (argv[2]) + 256, 1))) {
        printf ("Memory allocation error\n");
        gdbm_close (dbf);
        closedir (dir);
        return 0;
    }

    /* allocate memory for key */
    key.dsize = sizeof (gint)*3;
    key.dptr = malloc (key.dsize);

    k = (unsigned int*)key.dptr;

    while (entry = readdir (dir)) {
        /* we expect X_Y_Z.png files names */
        if (entry->d_name[0] == '.')
            continue;
        p = strchr (entry->d_name, '.');
        if (!p || strcmp (p, ".png"))
            continue;
        p = entry->d_name;
        if (sscanf (p, "%d%1c%d%1c%d", &x, &c1, &y, &c2, &z) < 5 || c1 != '_' || c2 != '_') {
            printf ("Name of %s is invalid, skiped\n", p);
            continue;
        }

        sprintf (name_buf, "%s/%s", argv[2], entry->d_name);
        if (stat (name_buf, &st)) {
            printf ("Cannot stat file %s, skipped\n", entry->d_name);
            continue;
        }

        fd = open (name_buf, O_RDONLY);
        if (!fd) {
            printf ("Cannot open file %s, skipped\n", entry->d_name);
            continue;
        }

        buf = (char*)malloc (st.st_size);
        if (!buf) {
            printf ("Buffer allocation error for file %s. Size %llu\n", entry->d_name, st.st_size);
            continue;
        }

        if (read (fd, buf, st.st_size) != st.st_size)
            printf ("Only part of data can be read from file %s. Skip file.\n", entry->d_name);
        else {
            /* fill key */
            k[0] = GINT_TO_BE (z);
            k[1] = GINT_TO_BE (x);
            k[2] = GINT_TO_BE (y);
            data.dsize = st.st_size;
            data.dptr = buf;
            if (!gdbm_store (dbf, key, data, GDBM_REPLACE))
                printf ("%d %d %d\n", x, y, z);
        }
        
        close (fd);
        free (buf);
    }
    
    closedir (dir);

    gdbm_close (dbf);
    free (key.dptr);
}
